@@ -303,7 +303,14 @@ namespace cil2ada.AssemblyParser
         while (countTokens > 0)
         {
           // create the corresponding object
-          this.genParameters.Add (new GenericParam (tokenRef[0]));
+          try
+          {
+              this.genParameters.Add(new GenericParam(tokenRef[0]));
+          } catch {
+              if (this.tokenKind == TokenKind.TypeRefResolved)
+                 Context.ReleaseContext();
+              throw;
+          }
           Context.importIntf.EnumGenericParams (ref hHandle, this.tokenscope, tokenRef, 1, out countTokens);
         }
         Context.importIntf.CloseEnum (hHandle);
@@ -493,16 +500,19 @@ namespace cil2ada.AssemblyParser
         case CorElementType.VALUETYPE:
           this.kind = TypeKind.ValueType;
           uint typeDefOrRef = Decoder.ReadTypeDefOrRefEncoded(sig, ref idx);
-          if (typeDefOrRef == 16777282)
+          Type auxt = Type.Get(typeDefOrRef, context);
+
+          if (auxt != null)
           {
-              this.initFromType
-                (Type.Get(typeDefOrRef, context));
+             this.initFromType(auxt);
           }
           else
           {
-              this.initFromType
-                (Type.Get(typeDefOrRef, context));
+              throw new System.ArgumentException
+                 ("Invalid ElementType during type signature analysis: 0x" 
+                  + b.ToString("x"));
           }
+
           break;
 
         case CorElementType.VAR:
